#include "controlwindowimpl.hpp"

ControlWindowImpl::ControlWindowImpl(MainWindow* mw, ControlWindow* cw)
: QObject(), mw(mw), cw(cw) {}

//
// Mandelbrot-Menge berechnen und Statusinformationen anpassen.
//
void ControlWindowImpl::renderMandelbrotImpl()
{
    //
    // Damit der Text sichtbar wird, muss er nicht nur gesetzt, sondern
    // auch gezeichnet (repaint()) werden. Damit dieses Event auch wirklich
    // stattfindet, wird seine Ausfuehrung mit processEvents() erzwungen.
    //
    cw->informationLabel->setText(tr("Mandelbrot-Menge wird berechnet..."));
    cw->informationLabel->repaint();
    qApp->processEvents();

    mandelbrot = new Mandelbrotmenge();

    newFractal = renderFractal(mandelbrot, "mandelbrot");

    cw->informationLabel->setText(tr("Mandelbrot-Menge wird angezeigt"));
    cw->informationLabel->repaint();
    qApp->processEvents();

    emit renderedFractal(newFractal);
}

//
// Mandelbar-Menge berechnen und Statusinformationen anpassen.
//
void ControlWindowImpl::renderMandelbarImpl()
{
    cw->informationLabel->setText(tr("Mandelbar-Menge wird berechnet..."));
    cw->informationLabel->repaint();
    qApp->processEvents();

    mandelbar = new Mandelbarmenge();

    newFractal = renderFractal(mandelbar, "mandelbar");

    cw->informationLabel->setText(tr("Mandelbar-Menge wird angezeigt"));
    cw->informationLabel->repaint();
    qApp->processEvents();

    emit renderedFractal(newFractal);
}

//
// Multibrot-Menge berechnen und Statusinformationen anpassen.
//
void ControlWindowImpl::renderMultibrotImpl()
{
    cw->informationLabel->setText(tr("Multibrot-Menge wird berechnet..."));
    cw->informationLabel->repaint();
    qApp->processEvents();

    double exponent = cw->multibrotSpinBox->value();

    multibrot = new Multibrotmenge();
    multibrot->setParam(std::complex<double>(exponent));

    newFractal = renderFractal(multibrot, "multibrot");

    cw->informationLabel->setText(tr("Multibrot-Menge wird angezeigt"));
    cw->informationLabel->repaint();
    qApp->processEvents();

    emit renderedFractal(newFractal);
}

//
// Julia-Menge berechnen und Statusinformationen anpassen.
//
void ControlWindowImpl::renderJuliaImpl()
{
    cw->informationLabel->setText(tr("Julia-Menge wird berechnet..."));
    cw->informationLabel->repaint();
    qApp->processEvents();

    double real = cw->juliaRealSpinBox->value();
    double imag = cw->juliaImagSpinBox->value();

    julia = new Juliamenge();
    julia->setParam(std::complex<double>(real, imag));

    newFractal = renderFractal(julia, "julia");

    cw->informationLabel->setText(tr("Julia-Menge wird angezeigt"));
    cw->informationLabel->repaint();
    qApp->processEvents();

    emit renderedFractal(newFractal);
}

//
// Zuruecksetzen aller Werte und Anzeigen der Mandelbrot-Menge.
//
void ControlWindowImpl::restoreDefaultsImpl()
{
    cw->multibrotSpinBox->setValue(2);
    cw->juliaRealSpinBox->setValue(0);
    cw->juliaImagSpinBox->setValue(0);
    cw->xSpinBox->setValue(0);
    cw->ySpinBox->setValue(0);
    cw->zoomSpinBox->setValue(0);
    cw->precisionSpinBox->setValue(50);
    cw->rgbRadioButton->setChecked(true);
    cw->optimizedCheckBox->setChecked(false);

    cw->informationLabel->setText(tr("Mandelbrot-Menge wird angezeigt"));
    cw->informationLabel->repaint();
    qApp->processEvents();

    mw->setDefaultFractal();
}

//
// Erzeugen des Fraktals anhand der im Fenster eingestellten Werte.
//
QImage ControlWindowImpl::renderFractal(ComplexFractal* fractal, std::string fractalName, bool show, int size)
{
    //
    // Ermitteln des gewaehlten Farbschemas.
    //
    if (cw->rgbRadioButton->isChecked())
    {
        mapper = new RGBColorMapper();
    }
    else if(cw->grayRadioButton->isChecked())
    {
        mapper = new GrayColorMapper();
    }
    else
    {
        mapper = new BWColorMapper();
    }

    fractalRenderer = new ComplexFractalRenderer(fractal, mapper);

    double zoom = cw->zoomSpinBox->value();

    int precision = cw->precisionSpinBox->value();

    double topLeftx = cw->xSpinBox->value();
    double topLefty = cw->ySpinBox->value();

    double step;

    //
    // Ermitteln der zum aktuellen Zoomfaktor passenden Schrittweite.
    //
    if (zoom <= 0)
    {
        step = (0.005 * (1 + std::abs(zoom) / 100));
    }
    else
    {
        step = (0.005 / (1 + zoom / 100));
    }

    //
    // Ermitteln des zur aktuellen Schrittweite passenden oberen linken Pixels.
    //
    std::complex<double> topLeft = std::complex<double>(topLeftx - step * 400, topLefty + step * 300);

    //
    // Dieser Wert wird ausschliesslich von den "Bild speichern unter..."-Funktionen
    // mit mittlerer und hoher Aufloesung auf false gesetzt, da diese das Fraktal neu
    // berechnen muessen. Das aktuell angezeigte soll davon nicht betroffen sein.
    //
    if (show)
    {
        mw->fractalLabel->topLeft = topLeft;
        mw->fractalLabel->name = fractalName;
    }

    bool optimized = false;

    if (cw->optimizedCheckBox->isChecked())
    {
        optimized = true;
    }

    //
    // Rendern des Fraktals unter Beruecksichtigung aller eingestellten Werte. Der
    // Parameter size wird ausschliesslich von den "Bild speichern unter..."-Funktionen
    // mit mittlerer und hoher Aufloesung auf einen anderen Wert als 1 gesetzt.
    //
    newFractal = fractalRenderer->render(topLeft, step / size, precision, 800 * size, 600 * size, optimized);

    delete fractalRenderer;

    return newFractal;
}

//
// Setzen eines neuen Mittelpunkt nach Mausklick im Fraktal-Bild.
//
void ControlWindowImpl::imageLabelClicked(int x, int y)
{
    double zoom = cw->zoomSpinBox->value();

    double step;

    //
    // Ermitteln der zum aktuellen Zoomfaktor passenden Schrittweite.
    //
    if (zoom <= 0)
    {
        step = (0.005 * (1 + std::abs(zoom) / 100));
    }
    else
    {
        step = (0.005 / (1 + zoom / 100));
    }

    //
    // Setzen der Koordinaten des Mittelpunktes unter Beruecksichtigung der
    // ermittelten Schrittweite und der empfangenen X- und Y-Werte des Mausklicks.
    //
    cw->xSpinBox->setValue(mw->fractalLabel->topLeft.real() + step * x);
    cw->ySpinBox->setValue(mw->fractalLabel->topLeft.imag() - step * y);
}
